/*******************************************************************************
dealcapture.cpp

example of an incremental subscription of FT_C_FILL class

*******************************************************************************/

#include "ftsome.h"
#include <string>
#include <map>

FTRevision appVer = {1,0,0};
unsigned long applSign = 732;

FTContext ft_ctx = NULL;
FTAPI_CONF_REC conf = {0};

static ClassPositionRec FT_C_FILL_Position = {0, 0, 0};

//METAMARKET CONNECTION DESCRIPTOR
FTConversation_DESC_REC mm_conv_desc;


/***************************************************************************
*  fill repo
**************************************************************************/

std::map<std::string, void*> fillMap;

long InitApplicationStructs(){
    return 0;
}

////////////////////////////////APPLICATION LOGIC ON SUBSCRIBED CLASSES ////////////////////////////////////////

long handleFT_C_FILL(long ConversationID, unsigned long ReqID, FTEventTypeEnum EventType, FTNotifySubscribeResStruct *ResStruct){
    FT_C_FILLPtr ftobj = (FT_C_FILLPtr)ResStruct->MarketObject;
    switch (ResStruct->EntityAction) {
        case FTEntityKIL:
            //set new class version
            UpdateVersion(ResStruct->TimeStamp[0], &FT_C_FILL_Position);
            SaveClassPosition("FT_C_FILL.pos", &FT_C_FILL_Position);
            if (ResStruct->KeyID <= 0 && !ResStruct->MarketObject){
                // This means a class reset at the server level.
                // Every entity received previously have to be invalidated!
                FTTraceAppl(FTLevel_Normal, "FTEntityKIL Action - Class Reset Signal - New version: %d", (ResStruct->TimeStamp)[0]);
            }
            else{
                FTTraceAppl(FTLevel_Normal, "FTEntityKIL Action - KeyID [%d]", ResStruct->KeyID);
            }
            break;
        case FTEntityDEL:
            FTTraceAppl(FTLevel_Normal, "EntityDEL Action - KeyID [%d]", ResStruct->KeyID);
            UpdateTimeStamp(ResStruct->TimeStamp, &FT_C_FILL_Position);
            SaveClassPosition("FT_C_FILL.pos", &FT_C_FILL_Position);
            break;
        case FTEntityADD:
        case FTEntityRWT:
            FTTraceAppl(FTLevel_Normal, "EntityADD/RWT Action - KeyID [%d]", ResStruct->KeyID);

            std::map<std::string, void*>::iterator it;
            std::string fid(ftobj->FillID);
            if((it = fillMap.find(fid)) != fillMap.end()){
                //we already have this fill
                FTTraceAppl(FTLevel_Normal, "mrk fill img unsol [%s] [%s] [%d]", ftobj->FillID, ftobj->User.OrderID, ftobj->Status);
                memcpy(it->second, ftobj, sizeof(FT_C_FILLRec));
            }else{
                FT_C_FILLPtr new_fill = (FT_C_FILLPtr)malloc(sizeof(FT_C_FILLRec));
                FTTraceAppl(FTLevel_Normal, "mrk fill ins [%s] [%s] [%d]", ftobj->FillID, ftobj->User.OrderID, ftobj->Status);
                memcpy(new_fill, ftobj, sizeof(FT_C_FILLRec));
                fillMap[fid] = (void*)new_fill;
            }
            UpdateTimeStamp(ResStruct->TimeStamp, &FT_C_FILL_Position);
            SaveClassPosition("FT_C_FILL.pos",&FT_C_FILL_Position);
            break;
    }
    return 0;
}



/////////////////////////////////SUBSCRIPTION CALLBACKS////////////////////////////////////////

static void NotifyClass(long ConversationID, unsigned long ReqID, FTEventTypeEnum EventType, FTNotifySubscribeResStruct *ResStruct){
    switch(EventType){
    case FTConnectionBroken:
        FTTraceAppl(FTLevel_Error, "onMMSubscribeNotify: connection broken (%d,%d)" , ConversationID, ReqID);
        break;
    case FTClassIdle:
        FTTraceAppl(FTLevel_Normal, "idle received on class %d", ResStruct->ClassID);
        break;
    case FTClassInstance:
        switch(ResStruct->ClassID){
            case FT_C_FILL_ID:
                handleFT_C_FILL(ConversationID, ReqID, EventType, ResStruct);
                break;
            default:
                break;
        }
    default:
        break;
    }
}

static void SubscribeResProc(long convID, unsigned long ReqID, long ResultCode, FTSubscribeResStruct *resStruct){
    FTTraceAppl(FTLevel_Normal, "-------- FTOpenSubscribeResProc: classID [%lu], SubscriptionID [%lu]", resStruct->ClassID, resStruct->SubscriptionID);
    FTTraceAppl(FTLevel_Normal, "-------- Result: %s", FTGetErrorString(ResultCode));

    if (ResultCode == FTOK){
        FTTraceAppl(FTLevel_Normal, "-------- Status: %s", (resStruct->SubscribeStatus==FTSubscribeStart)? "Start" : "Stop");
        FTTraceAppl(FTLevel_Normal, "-------- New Table version: %lu; To-Reset signal: %d", resStruct->ClassVersion, resStruct->ResetClass);

        if (resStruct->ResetClass){
            // In this case your request can't be served in incremental manner!
            // Any previous record received have to be considered invalidated and
            // the server will resend all the class content.
            switch(resStruct->ClassID){
                case FT_C_FILL_ID:
                    UpdateVersion(resStruct->ClassVersion, &FT_C_FILL_Position);
                    SaveClassPosition("FT_C_FILL.pos", &FT_C_FILL_Position);
                    break;
                default:
                    FTTraceAppl(FTLevel_Normal, "-------- Class id not managed - Class %d: New version [%lu]", resStruct->ClassID, resStruct->ClassVersion);
            }
            // Received data are stored into a file
            FTTraceAppl(FTLevel_Normal, "-------- Class Reset - Class %d: New version [%lu]", resStruct->ClassID, resStruct->ClassVersion);
        }
    }
}


/////////////////////////////////MM OPEN-CONV CALLBACKS////////////////////////////////////////
void MMOpenConvCallback(long ConversationID, long ResultCode, FTOpenResStruct *ResStruct){
    long res = 0;
    if((mm_conv_desc.open_conv_ResultCode = ResultCode) == FTOK){
       FTTraceAppl(FTLevel_Normal, "metamarket connection is ON!");
       FTTraceAppl(FTLevel_Normal, "Market Revision: V%d.%d.%d", ResStruct->MarketRevision[0], ResStruct->MarketRevision[1], ResStruct->MarketRevision[2]);

       mm_conv_desc.ConversationID = ConversationID;

       FTTraceAppl(FTLevel_Normal, "setting FT_C_FILL class version and timestamp for incremental subscription..");
       LoadClassPosition("FT_C_FILL.pos", &FT_C_FILL_Position);

       FTTraceAppl(FTLevel_Normal, "subscribing FT_C_FILL class..");
       res = FTStartSubscribeClassExt(ConversationID, 
                                        FTAckUnrequired,
                                        FTFlowAll,
                                        FT_C_FILL_ID,
                                        FT_C_FILL_Position.classVersion,
                                        FT_C_FILL_Position.TS,
                                        FTQueryAll,
                                        0,
                                        0, 0,
                                        SubscribeResProc,
                                        NotifyClass,
                                        777,
                                        0, 0);
    }else{
        FTTraceAppl(FTLevel_Normal, "metamarket connection FAILED! (%d)", ResultCode);
        if(ResultCode == FTInvalidCID){
            FTTraceAppl(FTLevel_Normal, "CID was already used on platform we try to connect with another one..");
                if((res = ConnectToServer(&conf, ft_ctx, 999)) < 0){
                FTTraceAppl(FTLevel_Normal, "ConnectToServer failed [result: %d] program exiting..", res);
                //error
                FTShutdown();
                exit(-1);
            }
        }
    }
}

void MMBrokenConvCallback(long ConversationID, FTBrokenTypeEnum BrokenType){
    FTTraceAppl(FTLevel_Error, "onMMBrokenConvCallback: connection broken (%d, %d)" , ConversationID, BrokenType);
}


/////////////////////////////////MAIN////////////////////////////////////////
int main(int argc, char *argv[]){
    long result = FTOK;

    if(LoadConfigByFile(&conf)){
        printf("PARAMETERS ERROR! exiting..\n");
        return 1;
    }

    //create an ft-context and set its properties
    ft_ctx = FTCreateContext();
    FTSetContextAttribute(ft_ctx, CTX_COMPRESSED, "ZIP");
    FTSetContextAttribute(ft_ctx, CTX_APPL_AUTHFILE, conf.license_file);

    //initialize ft-framework
    if(InitializeFTAPI(&conf, ft_ctx) != FTOK){
        //error
        FTShutdown();
        return 1;
    }

    conf.OpenResProc = MMOpenConvCallback;
    conf.BrokenProc = MMBrokenConvCallback;
    conf.UserType = FTUserTrader;

    InitApplicationStructs();

    //remember that connection will not be done until you call at least one FTRun() call.
    if(ConnectToServer(&conf, ft_ctx, 999) < 0){
        //error
        FTShutdown();
        return 1;
    }

/***********************************************************************************
** MAIN EVENT LOOP **
 Calling FTRun() ensures that all requests (subscriptions, transactions, queries ..) 
 will be handled by FTAPI durig program lifecycle.
***********************************************************************************/
    unsigned long times = 0; 
    do{
        if(FTRun() != FTOK){
            FTShutdown();
            return 1;
        }
        times++;
        if(!(times % 10)){
            FTTraceAppl(FTLevel_Full, "FTRun called %lu times", times);
        }
        FTSocketSelect();
    }while(1);

    //unreachable code
    return 0;
}


