#include "ftsome.h" 

/***************************************************************************
*  STATIC UTILS AND FUNCTIONS
**************************************************************************/

int IsCommentLine(const char *line){
	if(line[0] == '#'){
		return 1;
	}else if(line[0] == ' '){
		return IsCommentLine(&line[1]);
	}else{
		return 0;
	}
}

int IsBlankLine(const char *line){
	if(line[0] == '\n'){
		return 1;
	}else if(line[0] == ' '){
		return IsBlankLine(&line[1]);
	}else{
		return 0;
	}
}

void ResetClassPosition(ClassPositionPtr cp){
    cp->TS[0]=0;
    cp->TS[1]=0;
    cp->classVersion=0;
}

void PrintClassPosition(ClassPositionPtr cp){
    printf("[V:%lu]\n[T0:%lu]\n[T1:%lu]\n", cp->classVersion, cp->TS[0], cp->TS[1]);
}

int LoadClassPosition(char *filename, ClassPositionPtr cp){    
    FILE *f;
    ResetClassPosition(cp);

    if(f = fopen(filename,"r")){
        // Reads the class informations from a file
        fscanf(f, "%lu%lu%lu", &(cp->TS[0]), &(cp->TS[1]), &(cp->classVersion));
        fclose(f);
    }else{
        // If the file doesn't exist the class informations are set to zero
        cp->TS[0] = 0;
        cp->TS[1] = 0;
        cp->classVersion = 0;
    }
    PrintClassPosition(cp);
    return 0;
}

int SaveClassPosition(char *filename, const ClassPositionPtr cp){
    FILE *f;

    if(f = fopen(filename,"w")) {
        fprintf(f, "%lu\n%lu\n%lu\n", cp->TS[0], cp->TS[1], cp->classVersion);
        fclose(f);
        return 0;
    }else
        return -1;
}

/* New version from the server */
void UpdateVersion(unsigned long NewVersion, ClassPositionPtr cp){
    cp->classVersion = NewVersion;
    cp->TS[0] = 0;
    cp->TS[1] = 0;
}

/* New last-timestamp from the server for the class */
void UpdateTimeStamp(FTTimeStamp NewTS, ClassPositionPtr cp){
    if (FTCompareTS(NewTS, cp->TS) > 0 ){
        cp->TS[0] = NewTS[0];
        cp->TS[1] = NewTS[1];
    }
    else FTTraceAppl(FTLevel_Normal, "WARNING: time-stamp not progressive");
}


void traceCllbk(long Eventinfo, char* String){
    FTTraceLevelEnum tlvl = FTLevel_Undef;
    FTTraceSourceEnum src = FTSource_Unused;
    FTGetTraceInfo(Eventinfo, &src, &tlvl);
    if(src == FTSource_Application && tlvl >= FTLevel_Normal){
        printf("%s\n", String);
    }
}

//an int used to generate a pseudo-random cliendID
static unsigned int cid_ = 40001;

static unsigned int nextCID(){
    if(!cid_){
        cid_ = rand() % 65535;
    }else{
        cid_++;
    }
    return cid_;
}

/**
Performs FTRun() calls until a certain condition, denoted by a user function, is not verified.
**/
void FTRunCycleUntilCondition(EvalConditionFunction condition, void *ud){
    int i = 1;
    do{
        if(FTRun() != FTOK){
            printf("\n\nFTRun() **FAILED** program terminating.. \n\n");
            exit(-1);
        }
        FTSocketSelect();
    }while(!condition(ud));
}


void TransactionID_ToString(FTTransactionID *txid, char *stxid){
    sprintf(stxid, "%lu%lu%lu%lu%lu",
        txid->BusinessServiceID,
        txid->ClientServiceID, 
        txid->ClientID,
        txid->TimeStamp[0],
        txid->TimeStamp[1]);
}

void Dump_TX_Res_Info(long conversationID, unsigned long ReqID, FTSendTransactionResStruct *resStruct, char *desc){
    char stxid[CSTR_BUF_LEN];
    TransactionID_ToString(&resStruct->TransactionID, stxid);

    FTTraceAppl(FTLevel_Normal, "\n--Transaction Response--");
    FTTraceAppl(FTLevel_Normal, "@@%s@@", desc);
    FTTraceAppl(FTLevel_Normal, "  TXRES:         %d - %s", resStruct->TransactionStatus, resStruct->TransactionStatus == FTTransOK ? "TXOK" : "TXKO");
    FTTraceAppl(FTLevel_Normal, "  TXID:         %s", stxid);
    FTTraceAppl(FTLevel_Normal, "  ReqID:        %d", ReqID);
    FTTraceAppl(FTLevel_Normal, "  MarketReason: %d", resStruct->MarketReasonCode);
    FTTraceAppl(FTLevel_Normal, "  Class:        %lu", resStruct->ClassID);
    FTTraceAppl(FTLevel_Normal, "  KeyID:        %lu", resStruct->KeyID);
    FTTraceAppl(FTLevel_Normal, "  Action:       %d", (long)resStruct->EntityAction);

    if(resStruct->ResultClassID==FT_C_ERROR_INFO_ID && resStruct->ResultObject){
        FT_C_ERROR_INFOPtr info = (FT_C_ERROR_INFOPtr)resStruct->ResultObject;
        FTTraceAppl(FTLevel_Normal, "  MMReasonCode:   %lu", info->MMReasonCode);
        FTTraceAppl(FTLevel_Normal, "  MMErrorString:  %s", info->MMErrorString);
        FTTraceAppl(FTLevel_Normal, "  MrkReasonCode:   %lu", info->MarketReasonCode);
        FTTraceAppl(FTLevel_Normal, "  MrkErrorString:  %s", info->MarketErrorString);
    }
    FTTraceAppl(FTLevel_Normal, "------------------------\n");
}

//load a configuration
int LoadConfig(int argc, char *argv[], FTAPI_CONF_PTR conf){

    printf("---------------\n");
    printf("host: %s \n", argv[1]);
    printf("port: %s \n", argv[2]);
    printf("operator: %s \n", argv[3]);
    printf("password: %s \n", argv[4]);
    printf("license: %s \n", argv[5]);
    printf("serv name: %s \n", argv[6]);
    printf("---------------\n");

    strcpy(conf->host, argv[1]);
    conf->host_port = atoi(argv[2]);
    strcpy(conf->ftoperator, argv[3]);
    strcpy(conf->password, argv[4]);
    strcpy(conf->license_file, argv[5]);
    strcpy(conf->service_name, argv[6]);
    return 0;
}

int LoadConfigByFile(FTAPI_CONF_PTR conf, char *filename){
    FILE *f = fopen(filename, "r");
    if(f){
        fscanf(f, "%s%lu%s%s%s%s", conf->host,
                                &conf->host_port,
                                conf->ftoperator,
                                conf->password,
                                conf->license_file,
                                conf->service_name);

        printf("---------------\n");
        printf("host: %s \n", conf->host);
        printf("port: %lu \n", conf->host_port);
        printf("operator: %s \n", conf->ftoperator);
        printf("password: %s \n", conf->password);
        printf("license: %s \n", conf->license_file);
        printf("serv name: %s \n", conf->service_name);
        printf("---------------\n");

        fclose(f);
    }else{
        return 1;
    }
	if(conf->service_name[0] == '0')
		strcpy (conf->service_name,"");
    return 0;
}

int LoadConfigByFile(FTAPI_CONF_PTR conf){
    return LoadConfigByFile( conf, "lastconn");

}


//////////////////////////**** INITIALIZE FT FRAMEWORK ****/////////////////////////
long InitializeFTAPI(FTAPI_CONF_PTR conf_, FTContext ft_ctx_){
    long result = FTOK;

    int opt = FTTRACE_OPT_None;
    opt = opt | FTTRACE_OPT_APIInOut |
				FTTRACE_OPT_DataTransfert |
                FTTRACE_OPT_DataIn |
                FTTRACE_OPT_DataOut |
                FTTRACE_OPT_Application;

    result = FTSetTraceMode(NULL, FTLevel_Normal, opt, FTTrue, 0, 0);

    if (result != FTOK){
        printf("FTTrace not active");
    }

    //ftinit
    if((result = FTInitExt(traceCllbk, ft_ctx_)) != FTOK){
        return result;
    }

    /***************************************************************************
    DATAMODEL EXTENSION BEGIN
    ***************************************************************************/

    //metamarket classes loading
    if((result = FTExtendClassByLibrary(NULL, "ftlmetamarket", NULL)) != FTOK){
        FTTraceAppl(FTLevel_Error, "metamarket classes loading failed with error: %d", result);
        return result;
    }else{
        FTTraceAppl(FTLevel_Normal, "metamarket classes loaded!");
    }

    /***************************************************************************
    DATAMODEL EXTENSION END
    ***************************************************************************/

    //ftstart
    if((result = FTStart()) != FTOK){
        return result;
    }
    return result;
}

//////////////////////////**** SERVER CONNECTION ****/////////////////////////
long ConnectToServer(FTAPI_CONF_PTR conf, FTContext ft_ctx, unsigned int reqid){
    long result = 0;
    unsigned int cid = nextCID();
    //opening conversation.
    result = FTOpenConversationExt( conf->host,
                                    conf->host_port,
                                    conf->UserType,
                                    cid,
                                    conf->ftoperator,
                                    conf->password,
                                    appVer,
                                    applSign,
                                    conf->OpenResProc,
                                    conf->BrokenProc,
                                    reqid,
								    strcmp(conf->service_name, "") ? conf->service_name : 0,
                                    ft_ctx);
    return result;
}
